uniform sampler2D colortexture;
uniform sampler2D envtexture;
uniform sampler2D doblurtexture;
uniform sampler2D dpsblurtexture;
uniform sampler2D bounceblurtexture;
uniform sampler2D lighttexture;
uniform sampler2D lightspectexture;

uniform sampler2D ndtexture;
uniform sampler2D ndtextureback;
uniform sampler2D positiontexture;

uniform int maximum;
uniform vec2 viewport;
uniform int dpsSampleCount;
uniform bool computeShadowMap;

void shadowCompute(inout float color);

vec2 textureCoord(const vec2 coord){	
	return coord/viewport;
}

vec4 getColor(in vec2 coord){
	return texture( colortexture, coord );
}

float getAOFrag(in vec2 coord){
	return texture( doblurtexture, coord ).w;
}

vec4 getEnv(in vec2 coord){
	return vec4(texture( doblurtexture, coord ).xyz,1.0);
}

vec4 getBounce(in vec2 coord){
	return texture( bounceblurtexture, coord );
}

vec4 getLight(in vec2 coord){
	return texture( lighttexture, coord );
}

vec4 getSpecLight(in vec2 coord){
	return texture( lightspectexture, coord );
}

float getDpsMask(in vec2 coord){
	return texture( dpsblurtexture, coord ).w;
}

vec4 getDpsColor(in vec2 coord){
	return vec4(texture( dpsblurtexture, coord).xyz,1.0);
}

vec4 getSumOfLight(){
	vec3 sum = vec3(0.0);
	for (int i=0; i<maximum;i++)
	{
		sum += gl_LightSource[i].diffuse.xyz;
	}
	return vec4(sum,1.0);
}

float getShadows(in vec2 coord, inout vec4 dpsColor, in float dpsMask){
	if (maximum<1){
		return 1.0;
	}else{
		//if (computeShadowMap && maximum < 2) {
		if (computeShadowMap) {
			float shadowColor = 1.0;
			shadowCompute(shadowColor);
			dpsColor = vec4(0.0);
			return 1.0-shadowColor;
		}else{
			if (dpsSampleCount>0){
				return dpsMask;
			}else{
				return 0.0;
			}
		}
	}	
}

///////////////////////////////////////////////////////////

void main(void)
{
	vec2 fragCoord = textureCoord(gl_FragCoord.xy);

	vec4 finalColor;
	vec4 textureColor = getColor(fragCoord);
	
	float doShadows = getAOFrag(fragCoord);
	vec4 doBounce = getBounce(fragCoord);
	//vec4 envColor = getEnv(fragCoord);
	vec4 envColor = 2.0*getEnv(fragCoord);

	vec4 lightColor = getLight(fragCoord);
	vec4 lightSpecColor = getSpecLight(fragCoord);
	
	vec4 dpsColor = getDpsColor(fragCoord);
	float dpsMask = getDpsMask(fragCoord);

	float shadows = getShadows(fragCoord,dpsColor,dpsMask);

	//final composition		
	vec4 lightSum = getSumOfLight();
	float lightMean = (lightSum.x + lightSum.y + lightSum.z) / 3.0;
	float lightCoef = 1.0/(lightMean+1.0);

	// ( lightColorInShadows + lightColorInDirectLight ) * textureColor + additionalLightEffects
	finalColor = shadows * doShadows * (envColor+dpsColor) * lightCoef;
	finalColor += (1.0-shadows) * (lightColor+envColor*lightCoef);
	finalColor *= textureColor;
	finalColor += doBounce;
	finalColor += lightSum*0.1;
	finalColor += (1.0-shadows) * lightSpecColor * gl_FrontMaterial.specular;
	finalColor += gl_FrontMaterial.emission;
			
	gl_FragColor = vec4(finalColor.xyz, gl_Color.a*gl_FrontMaterial.diffuse.a);
}
